/*
 * Copyright (C) 2004, 2005 Michael Dominic K. (http://www.mdk.org.pl)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using Gdk;
using Gtk;
using Gnome;
using MovieDatabase;
using System;
using Glade;
using Util.Windows;
using Util;

public class EditMovie_Window {

    /*Translatable strings */
    static readonly string Msg_NoTitle = "You have to provide at least the title of the movie!";
    static readonly string Msg_Runtime = "Runtime is not a valid integer value (minutes)";
    static readonly string Msg_Year = "Year is not a valid integer value (ie. 1996)";
   
    //////////////////////////////////////////////////
    
    Movie EditedMovie = null;

    [Widget] Gtk.Window Window_Add_GtkWindow;

    [Widget] Gtk.SpinButton No_GtkSpinButton;
    [Widget] Gtk.Entry Title_GtkEntry;
    [Widget] Gtk.Entry TranslatedTitle_GtkEntry;
    [Widget] Gtk.Entry Runtime_GtkEntry;
    [Widget] Gtk.Entry Year_GtkEntry;
    [Widget] Gtk.Entry Quality_GtkEntry;
    
    [Widget] Gtk.RadioButton _1_GtkRadioButton;
    [Widget] Gtk.RadioButton _2_GtkRadioButton;
    [Widget] Gtk.RadioButton _3_GtkRadioButton;
    [Widget] Gtk.RadioButton _4_GtkRadioButton;

    [Widget] Gtk.TextView Description_GtkTextView;
    [Widget] Gtk.Button IMDBSearch_GtkButton;
    [Widget] Gtk.Table Grid_GtkTable;

    // Combos
    Gtk.ComboCached Director_GtkComboCached;
    Gtk.ComboCached Country_GtkComboCached;
    Gtk.ComboCached Genre_GtkComboCached;

    Gtk.ComboCached VideoFormat_GtkComboCached;
    Gtk.ComboCached AudioFormat_GtkComboCached;
    Gtk.ComboCached Release_GtkComboCached;
    Gtk.ComboCached Subtitles_GtkComboCached;
    Gtk.ComboCached Regions_GtkComboCached;
   
    public void CreateWidgets() 
    {
        Glade.XML gxml = new Glade.XML (null, "AddWindow.glade", "Window_Add_GtkWindow", null);
        gxml.Autoconnect (this);

        // Icon
        Window_Add_GtkWindow.Icon = new Pixbuf(null, "windowicon.png");

        No_GtkSpinButton.Value = MovieDB.CheckMaxNo() + 1;
        
        // Create cached controls
        Director_GtkComboCached = new Gtk.ComboCached(Global.Directors_Cache);
        Country_GtkComboCached = new Gtk.ComboCached(Global.Countrys_Cache);
        Genre_GtkComboCached = new Gtk.ComboCached(Global.Genres_Cache);
        // Media
        VideoFormat_GtkComboCached = new Gtk.ComboCached(Global.VFormats_Cache);;
        AudioFormat_GtkComboCached = new Gtk.ComboCached(Global.AFormats_Cache);;
        Release_GtkComboCached = new Gtk.ComboCached(Global.Releases_Cache);;
        Subtitles_GtkComboCached = new Gtk.ComboCached(Global.Subtitles_Cache);;
        Regions_GtkComboCached = new Gtk.ComboCached(Global.Regions_Cache);;
        Regions_GtkComboCached.Sensitive = false;
        // By default we're insensitive. Setting the combo might change that 
        // (On_VideoFormatChanged)

        // Attach events
        VideoFormat_GtkComboCached.Entry.Changed += new EventHandler(On_VideoFormatChanged);


        // Pack
        Grid_GtkTable.Attach(Director_GtkComboCached, 1, 2, 4, 5);
        Grid_GtkTable.Attach(Country_GtkComboCached, 1, 2, 5, 6);
        Grid_GtkTable.Attach(Genre_GtkComboCached, 1, 2, 6, 7);

        Grid_GtkTable.Attach(VideoFormat_GtkComboCached, 1, 2, 9, 10);
        Grid_GtkTable.Attach(AudioFormat_GtkComboCached, 1, 2, 10, 11);
        Grid_GtkTable.Attach(Regions_GtkComboCached, 1, 2, 11, 12);
        Grid_GtkTable.Attach(Release_GtkComboCached, 1, 2, 13, 14);
        Grid_GtkTable.Attach(Subtitles_GtkComboCached, 1, 2, 14, 15);
                
    }
        
    /* We need to look if VideoFrmat contains DVD and activate/deactivate
       the DVD regions widget */
    public void On_VideoFormatChanged (object o, EventArgs args)
    {
        if (VideoFormat_GtkComboCached.Entry.Text.IndexOf("DVD") != -1) {
            Regions_GtkComboCached.Sensitive = true;
        } else { 
            Regions_GtkComboCached.Entry.Text = "";
            Regions_GtkComboCached.Sensitive = false;
        }
    }

    public void FillWidgets(Movie _M)
    {
        if (_M.Data[(int)MovieField.NO] != null) No_GtkSpinButton.Value = Convert.ToInt32(_M.Data[(int)MovieField.NO]);
        
        if (_M.Data[(int)MovieField.TITLE] != null) Title_GtkEntry.Text = _M.Data[(int)MovieField.TITLE];
        if (_M.Data[(int)MovieField.TRANSLATEDTITLE] != null) TranslatedTitle_GtkEntry.Text = _M.Data[(int)MovieField.TRANSLATEDTITLE];
        if (_M.Data[(int)MovieField.YEAR] != null) Year_GtkEntry.Text = _M.Data[(int)MovieField.YEAR];
        if (_M.Data[(int)MovieField.TIME] != null) Runtime_GtkEntry.Text = _M.Data[(int)MovieField.TIME];
        if (_M.Data[(int)MovieField.DIRECTOR] != null) Director_GtkComboCached.Entry.Text = _M.Data[(int)MovieField.DIRECTOR];
        if (_M.Data[(int)MovieField.COUNTRY] != null) Country_GtkComboCached.Entry.Text = _M.Data[(int)MovieField.COUNTRY];
        if (_M.Data[(int)MovieField.GENRE] != null) Genre_GtkComboCached.Entry.Text = _M.Data[(int)MovieField.GENRE];
        
        if (_M.Data[(int)MovieField.DISCS] != null) {
            if (_M.Data[(int)MovieField.DISCS] == "1") _1_GtkRadioButton.Active = true; else
                if (_M.Data[(int)MovieField.DISCS] == "2") _2_GtkRadioButton.Active = true; else
                    if (_M.Data[(int)MovieField.DISCS] == "3") _3_GtkRadioButton.Active = true; else
                        _4_GtkRadioButton.Active = true ;
        }
                
        if (_M.Data[(int)MovieField.VIDEOFORMAT] != null) VideoFormat_GtkComboCached.Entry.Text = _M.Data[(int)MovieField.VIDEOFORMAT];
        if (_M.Data[(int)MovieField.AUDIOFORMAT] != null) AudioFormat_GtkComboCached.Entry.Text = _M.Data[(int)MovieField.AUDIOFORMAT];
        if (_M.Data[(int)MovieField.DVDREGION] != null) Regions_GtkComboCached.Entry.Text = _M.Data[(int)MovieField.DVDREGION];

        if (_M.Data[(int)MovieField.QUALITY] != null) Quality_GtkEntry.Text = _M.Data[(int)MovieField.QUALITY];
        if (_M.Data[(int)MovieField.RELEASE] != null) Release_GtkComboCached.Entry.Text = _M.Data[(int)MovieField.RELEASE];
        if (_M.Data[(int)MovieField.SUBTITLES] != null) Subtitles_GtkComboCached.Entry.Text = _M.Data[(int)MovieField.SUBTITLES];
        
        if (_M.Data[(int)MovieField.DESCRIPTION] != null) Description_GtkTextView.Buffer.Text = _M.Data[(int)MovieField.DESCRIPTION];
    }

    public void On_TitleEntryChanged (object o, EventArgs args) 
    {
        // FIXME: Some spaces trimming
        if (Title_GtkEntry.Text != "" && Title_GtkEntry.Text != " ") IMDBSearch_GtkButton.Sensitive = true;
        else IMDBSearch_GtkButton.Sensitive = false;

        
    }

    /* Check if the widgets are filled properly */
    public bool CheckWidgets()
    {
        // FIXME We need some better validations here (year, etc.) 
        if (!Widgets.ValidateNotEmptyTextEntry(Title_GtkEntry)) {
            new OKDialogWindow(Msg_NoTitle, Window_Add_GtkWindow, MessageType.Warning);
            return false;
        }
        
        if (Runtime_GtkEntry.Text != String.Empty && !Widgets.ValidateIntegerTextEntry(Runtime_GtkEntry)) {
            new OKDialogWindow(Msg_Runtime, Window_Add_GtkWindow, MessageType.Warning);
            return false;
        }
        
        if (Year_GtkEntry.Text != String.Empty && !Widgets.ValidateIntegerTextEntry(Year_GtkEntry)) {
            new OKDialogWindow(Msg_Year, Window_Add_GtkWindow, MessageType.Warning);
            return false;
        }

        // All ok
        return true;
    }
    

    /* Fill the movie structure according to widgets */
    public Movie FetchWidgets(Movie __M)
    {
        if (__M == null) __M = new Movie();
        
        // Fill in the values
        __M.Data[(int)MovieField.NO] = Convert.ToString(No_GtkSpinButton.Value);
        __M.Data[(int)MovieField.TITLE] = Title_GtkEntry.Text;
        __M.Data[(int)MovieField.TRANSLATEDTITLE] = TranslatedTitle_GtkEntry.Text;
        __M.Data[(int)MovieField.TIME] = Runtime_GtkEntry.Text;
        __M.Data[(int)MovieField.YEAR] = Year_GtkEntry.Text;
        __M.Data[(int)MovieField.DIRECTOR] = Director_GtkComboCached.Entry.Text;
        __M.Data[(int)MovieField.COUNTRY] = Country_GtkComboCached.Entry.Text;
        __M.Data[(int)MovieField.GENRE] = Genre_GtkComboCached.Entry.Text;
        
        string __Discs;
        if (_1_GtkRadioButton.Active == true) __Discs = "1"; else
            if (_2_GtkRadioButton.Active == true) __Discs = "2"; else 
                if (_3_GtkRadioButton.Active == true) __Discs = "3"; else 
                    __Discs = "4";
        
        __M.Data[(int)MovieField.DISCS] = __Discs;
        __M.Data[(int)MovieField.VIDEOFORMAT] = VideoFormat_GtkComboCached.Entry.Text;
        __M.Data[(int)MovieField.AUDIOFORMAT] = AudioFormat_GtkComboCached.Entry.Text;
        __M.Data[(int)MovieField.DVDREGION] = Regions_GtkComboCached.Entry.Text;
        __M.Data[(int)MovieField.QUALITY] = Quality_GtkEntry.Text;
        __M.Data[(int)MovieField.RELEASE] = Release_GtkComboCached.Entry.Text;
        __M.Data[(int)MovieField.SUBTITLES] = Subtitles_GtkComboCached.Entry.Text;
        
        __M.Data[(int)MovieField.DESCRIPTION] = Description_GtkTextView.Buffer.Text;
        return __M;
    }
    
    // *** Constructor (New Movie)
    public EditMovie_Window() 
    {
        CreateWidgets();
        Window_Add_GtkWindow.ShowAll();
        EditedMovie = new Movie();
        EditedMovie.Data[(int)MovieField.ID] = null;

        // Set focus to title
        Title_GtkEntry.GrabFocus();
    }


    // *** Constructor (Edit Movie)
    public EditMovie_Window(Movie _Movie) 
    {
        CreateWidgets();
        EditedMovie = _Movie;
        FillWidgets(_Movie);
        Window_Add_GtkWindow.Title = "Edit Movie";
        Window_Add_GtkWindow.ShowAll();
    }
    

    public Movie Run() 
    {
        Application.Run();
        return EditedMovie;
    }

    public void On_Cancel (object o, EventArgs args) 
    {
        KillWindow(false);
    }

    public void On_OK (object o, EventArgs args)
    {
        // TODO: Strip Spaces

        if (!CheckWidgets()) {
            Debug.Message("Monotheka.exe", "EditMovie_Window", "Failed validating widgets");
        } else {
            EditedMovie = FetchWidgets(EditedMovie);

            // We don't need to changed dates for edited movie
            if (EditedMovie.Data[(int)MovieField.ADDEDDATE] == null || EditedMovie.Data[(int)MovieField.ADDEDDATE] == String.Empty ) EditedMovie.Data[(int)MovieField.ADDEDDATE] = DateTime.Now.ToFileTime().ToString();

            // Update Cache Values
            // We need to update cache ONLY if it's not a "group" work or production
            // FIXME We should actually split the ','
            if (Director_GtkComboCached.Entry.Text.IndexOf(',') == -1) Director_GtkComboCached.AddCurrent();
            if (Country_GtkComboCached.Entry.Text.IndexOf(',') == -1) Country_GtkComboCached.AddCurrent();
            Genre_GtkComboCached.AddCurrent();
            
            VideoFormat_GtkComboCached.AddCurrent();
            AudioFormat_GtkComboCached.AddCurrent();
            Regions_GtkComboCached.AddCurrent();
            Release_GtkComboCached.AddCurrent();
            Subtitles_GtkComboCached.AddCurrent();
            
            KillWindow(true);
        }
    }

    public void KillWindow(bool _ret)
    {
        if (_ret == false) EditedMovie = null;
        Window_Add_GtkWindow.Destroy();
        Application.Quit();
    }
    public void On_WindowDelete (object o, DeleteEventArgs args) 
    {
        KillWindow(false);
    }

    public void On_IMDBSearch (object o, EventArgs args)
    {
        if (Title_GtkEntry.Text != "") {
            try {
                Util.Net.LaunchWebbrowser("http://www.imdb.com/find?q=" + Title_GtkEntry.Text + ";tt=on;nm=off;mx=5");
            }
            catch (NetworkException __NExcp) {
                new OKDialogWindow(__NExcp.Message, Window_Add_GtkWindow, MessageType.Warning);
            }
        }
        

            
    }

    public void On_GetFrom (object o, EventArgs args)
    {
        Movie __M = FetchWidgets(EditedMovie);
        GetFrom_Window __Win = new GetFrom_Window(__M);
        if (__Win.GetMovie() != null) FillWidgets(__Win.GetMovie());
    }

}

