/*
 * Copyright (C) 2004, 2005 Michael Dominic K. (http://www.mdk.org.pl)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* This class is responsible for monitoring the GConf registry and
 * reacting to changes (calling refresh functions, etc)
 */

using System;
using GConf;
using Util;


public class GConfMonitor 
{
    
    /* Load all the (initial) settings */
    public static void LoadSettings()
    {
        Debug.Message("Monotheka.exe", "GConfMonitor", "Loading all the setting from the GConf registry");
        
        ApplicationSettings.NewColor = (string) GConfRegistry.Get(GConfKeys.NewColor, GConfKeys.NewColor_Default);
        ApplicationSettings.BorrowedColor = (string) GConfRegistry.Get(GConfKeys.BorrowedColor, GConfKeys.BorrowedColor_Default);
        ApplicationSettings.OverBorrowedColor = (string) GConfRegistry.Get(GConfKeys.OverBorrowedColor, GConfKeys.OverBorrowedColor_Default);
        
        ApplicationSettings.DaysNew = (int) GConfRegistry.Get(GConfKeys.DaysNew, GConfKeys.DaysNew_Default);
        ApplicationSettings.DaysOverBorrowed = (int) GConfRegistry.Get(GConfKeys.DaysOverBorrowed, GConfKeys.DaysOverBorrowed_Default);
    }


    /* Initialize, register notifications */
    public static void Init()
    {
        Debug.Message("Monotheka.exe", "GConfMonitor", "Initializing monitor, setting notifications");
    
        // Register all the notification for the main movie view
        // FIXME: Only one "new"
        // FIXME: Maybe we could use on functions, but with ONE new NotifyEventHandler? 
        GConfRegistry.AddNotify(GConfKeys.NewColor, new NotifyEventHandler(OnNewColorChanged));
        GConfRegistry.AddNotify(GConfKeys.BorrowedColor, new NotifyEventHandler(OnBorrowedColorChanged));
        GConfRegistry.AddNotify(GConfKeys.OverBorrowedColor, new NotifyEventHandler(OnOverBorrowedColorChanged));
        GConfRegistry.AddNotify(GConfKeys.DaysNew, new NotifyEventHandler(OnDaysNewChanged));
        GConfRegistry.AddNotify(GConfKeys.DaysOverBorrowed, new NotifyEventHandler(OnDaysOverBorrowedChanged));

    }

    public static void OnNewColorChanged (object _Sender, NotifyEventArgs _Args)
    {
        ApplicationSettings.NewColor = (string) _Args.Value;
        RefreshViews();
    }

    
    public static void OnBorrowedColorChanged (object _Sender, NotifyEventArgs _Args)
    {
        ApplicationSettings.BorrowedColor = (string) _Args.Value;
        RefreshViews();
    }


    public static void OnOverBorrowedColorChanged (object _Sender, NotifyEventArgs _Args)
    {
        ApplicationSettings.OverBorrowedColor = (string) _Args.Value;
        RefreshViews();
    }


    public static void OnDaysNewChanged (object _Sender, NotifyEventArgs _Args)
    {
        ApplicationSettings.DaysNew = (int) _Args.Value;
        RefreshViews();
    }


    public static void OnDaysOverBorrowedChanged (object _Sender, NotifyEventArgs _Args)
    {
        ApplicationSettings.DaysOverBorrowed = (int) _Args.Value;
        RefreshViews();
    }


    /* Changes to Main Window */
    public static void RefreshViews ()
    {
        Debug.Message("Monotheka.exe", "GConfMonitor", "Notification received, refreshing data&movie views");
       
        // Let's refresh our main view
        Window_Main.MovieDataView.RefreshView(true);
        Window_Main.MovieMovieView.Refresh();
    }


}
