/*
 * Copyright (C) 2004, 2005 Michael Dominic K. (http://www.mdk.org.pl)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using System.Data;
using Mono.Data.SqliteClient;
using System.Collections;
using Util;
using System.IO;

namespace MovieDatabase 
{
    public enum DataFieldType { TEXT, NUMBER, PIXBUF, TIMESTAMP }

    public class DBInfo {
        public string Name;
        public string Owner;
        public string Contact;
        public string Comment;
        public int Version;

        public static bool operator != (DBInfo _A, DBInfo _B)
        {
            // FIXME: Version checking? 
            if (_A.Name != _B.Name       ||
                _A.Owner != _B.Owner     ||
                _A.Contact != _B.Contact ||
                _A.Comment != _B.Comment) return true;
            return false;
        }

        public static bool operator == (DBInfo _A, DBInfo _B)
        {
            if (_A != _B) return false;
            return true;
        }

        /* Just to kill some warnings */
        public override bool Equals(object _A)
        {
            if (this == (DBInfo)_A) return true;
            else return false;
        }

        /* Just to kill some warnings */
        public override int GetHashCode()
        {
            return Name.GetHashCode() + Owner.GetHashCode() + Contact.GetHashCode() + Comment.GetHashCode() + Version.GetHashCode();

        }
        
    }
             

    public class DataField
    {
        public string Name = "";
        public string SqlName = "";
        public DataFieldType Type;
        
        public bool Searchable;
        public bool Importable;
        public bool Columnable;
        
        public DataField() 
        {
            Name = "";
            SqlName = "";
            Searchable = true;
            Importable = true;
            Columnable = true;
            Type = DataFieldType.TEXT;
        }

    }

    public class ColumnsRequest
    {
        public bool[] Columns = new bool[21];

        public override string ToString()
        {
            string __Ret = "";
            
            for (int __i = 0; __i < Columns.Length; __i++) 
                if (Columns[__i] == true) __Ret += ((DataField)MovieDB.Fields[__i]).Name + ",";
            
            __Ret = __Ret.TrimEnd(',');
            return __Ret;

        }

        public static ColumnsRequest operator +(ColumnsRequest _A, MovieField _B)
        {
            _A.Columns[(int) _B] = true;
            return _A;
        }

        public static ColumnsRequest operator +(ColumnsRequest _A, ColumnsRequest _B)
        {
            ColumnsRequest __C = new ColumnsRequest();

            for (int __i = 0; __i < _A.Columns.Length; __i++)
                if (_B.Columns[__i] == true || _A.Columns[__i] == true) __C.Columns[__i] = true;
                else __C.Columns[__i] = false;

            return __C;
        }

        public static ColumnsRequest operator -(ColumnsRequest _A, MovieField _B)
        {
            _A.Columns[(int) _B] = false;
            return _A;
        }

        public void AllColumns() 
        {
            for (int __i = 0; __i < Columns.Length; __i++)
                Columns[__i] = true;
        }

        public ColumnsRequest()
        {
            for (int __i = 0; __i < Columns.Length; __i++)
                Columns[__i] = false;
                
        }

        public ColumnsRequest(string _From)
        {
            // Reset
            for (int __i = 0; __i < Columns.Length; __i++)
                Columns[__i] = false;

            string[] __S = _From.Split(',');
            
            foreach (string __ClmnName in __S) {
                int __i = 0;
                foreach (DataField __DF in MovieDB.Fields) {
                    if (__DF.Name == __ClmnName) { Columns[__i] = true;
                    break;
                    }
                    __i++;
                }
                   
            }
            
        }




        public ColumnsRequest(params MovieField[] _Clmns)
        {
            foreach (MovieField __F in _Clmns)
                Columns[(int)__F] = true;
                
        }

    }
    
    
    public class MovieDB
    {
        public static IDbConnection DBConnection;
        public static ArrayList Fields = new ArrayList();
        private static ColumnsRequest MandatoryColumns = new ColumnsRequest();
        public static bool FreshDatabase = false;
        
        /* Get DBInfo structure that contains all the personal 
           information about this database */
        public static DBInfo GetDBInfo()
        {
            string __query = "SELECT Name, Owner, Contact, Comment, Version FROM DBInfo";
            
            // Create&Execute the command
            // FIXME: That should be moved to some kind of utils
            IDbCommand Cmd = DBConnection.CreateCommand();
            Cmd.CommandText = __query;
            IDataReader Reader = Cmd.ExecuteReader();
            
            // Read
            Reader.Read();
            
            if (Reader.FieldCount == 0) throw new Exception("Can't find database information! Aborting...");

            DBInfo __Inf = new DBInfo();
            __Inf.Name = Reader.GetString(0);
            __Inf.Owner = Reader.GetString(1);
            __Inf.Contact = Reader.GetString(2);
            __Inf.Comment = Reader.GetString(3);
            __Inf.Version = Reader.GetInt32(4);

            // clean up
            Reader.Close();
            Reader = null;
            Cmd.Dispose();
            Cmd = null;
            
            return __Inf;
        }


        /* Update DB params */
        public static void PutDBInfo(DBInfo __DB)
        {
            string __query = String.Format("UPDATE DBInfo SET Name='{0}', Owner='{1}', Contact='{2}', Comment='{3}', Version='{4}'", 
                                           Util.Strings.DBLize(__DB.Name), 
                                           Util.Strings.DBLize(__DB.Owner), 
                                           Util.Strings.DBLize(__DB.Contact), 
                                           Util.Strings.DBLize(__DB.Comment), 
                                           Util.Strings.DBLize(__DB.Version.ToString()));
            ExecuteQuery(__query);
        }
        

        public static void InitFields()
        {
            // Create all the names
            string[] __Names = new string[] {	"ID", "Flags", "No", "Title", "Translated Title", "Director", "Time",
                                                "Year", "Country", "Genre", "Discs", "Video Format", "Audio Format", 
                                                "Quality", "Subtitles", "Release", "Borrower", "Description", 
                                                "Added Date", "Borrowed Date" , "DVD Region" };
            
            // NOTE: DO NOT TRANSLATE THAT!
            string[] __SqlNames = new string[] {"ID", "Flags" , "No", "Title", "TranslatedTitle", "Director", "Time", 
                                                "Year", "Country", "Genre", "Discs", "VideoFormat", "AudioFormat", 
                                                "Quality", "Subtitles", "Release", "Borrower", "Description",
                                                "AddedDate" , "BorrowedDate" , "DVDRegion" };
            
            // Create DATA FIELDS         
            for (int __i = 0; __i < __Names.Length ; __i++)
              {
                  DataField DF = new DataField();
                  
                  DF.Name = __Names[__i];
                  DF.SqlName = __SqlNames[__i];
                  Fields.Add(DF);
              }
            
            ((DataField)Fields[0]).Type = DataFieldType.NUMBER;
            ((DataField)Fields[1]).Type = DataFieldType.PIXBUF;
            ((DataField)Fields[2]).Type = DataFieldType.NUMBER;
            ((DataField)Fields[6]).Type = DataFieldType.NUMBER;
            ((DataField)Fields[7]).Type = DataFieldType.NUMBER;
            ((DataField)Fields[10]).Type = DataFieldType.NUMBER;
            ((DataField)Fields[18]).Type = DataFieldType.TIMESTAMP;
            ((DataField)Fields[19]).Type = DataFieldType.TIMESTAMP;
            	
    	    ((DataField)Fields[0]).Importable = false;
            ((DataField)Fields[1]).Importable = false;
            ((DataField)Fields[18]).Importable = false;
            ((DataField)Fields[19]).Importable = false;
            
            ((DataField)Fields[0]).Searchable = false;
            ((DataField)Fields[1]).Searchable = false;
            ((DataField)Fields[6]).Searchable = false;
            ((DataField)Fields[10]).Searchable = false;
            ((DataField)Fields[17]).Searchable = false;
            ((DataField)Fields[18]).Searchable = false;
            ((DataField)Fields[19]).Searchable = false;
                        
            ((DataField)Fields[0]).Columnable = false;
            ((DataField)Fields[17]).Columnable = false;
            
        }


        /* Returns true if fresh database was created */
        public static bool CreateDatabase()
        {
            // Create table string
            string __MoviesQuery = "CREATE TABLE Movies (ID INTEGER PRIMARY KEY,";
            foreach (DataField __Field in Fields) 
                {
                    if (__Field.Name != "ID" ) 
                        if (__Field.Type != DataFieldType.NUMBER) __MoviesQuery += __Field.SqlName + " TEXT,";
                        else __MoviesQuery += __Field.SqlName + " INT,";
                }
              
            __MoviesQuery = __MoviesQuery.TrimEnd(',');
            __MoviesQuery += ")";
            
            // Create DBINfo query
            string __DBInfoQuery = "CREATE TABLE DBInfo (Name TEXT, Owner TEXT, Contact TEXT, Comment TEXT, Version TEXT)";
            string __DBInfoInsert = "INSERT INTO DBInfo (Name, Owner, Contact, Comment, Version) VALUES ('your database name', 'database owner' , 'e-mail or phone', 'additional notes' , '5')";
            
            
            try {
                Debug.Message("MovieDatabase.dll", "MovieDB", "Creating Movies table, QUERY=" + __MoviesQuery);
                ExecuteQuery(__MoviesQuery);
                Console.WriteLine("Creating fresh database, version {0}", "5");
            }
            catch {
                return false;
            }
              
            // If we managed to create the Movies table, we HAVE TO manage to create the rest
            
            try {
                ExecuteQuery(__DBInfoQuery);
                ExecuteQuery(__DBInfoInsert);
            } catch (Exception _Excp) {
                throw new Exception("Error during fresh database creation!", _Excp);
            }
            
            return true;
        }



        public static void InitDB(string filename)
        {
            Debug.Message("MovieDatabase.dll", "MovieDB", "Initializing database");            

            Debug.Message("MovieDatabase.dll", "MovieDB", "Checking if file exists");
            FreshDatabase = ! (File.Exists(filename));
                        
            DBConnection = new SqliteConnection("URI=file:" + filename);
            DBConnection.Open();
            if (DBConnection == null) throw new Exception("Can't open the DBConnection! SQLLite not present?");
            
            if (!FreshDatabase) {
                
                // We need to upgrade...
                DBInfo __DBInfo;
                try {
                    __DBInfo = GetDBInfo();
                } catch {
                    // Assuming version without DBInfo (1)
                    __DBInfo.Version = -1;
                }
                
                Debug.Message("MovieDatabase.dll", "MovieDB", String.Format("Database version is {0}", __DBInfo.Version));
                
                // -1 to 0
                if (__DBInfo.Version < 0) {
                    Console.WriteLine("Upgrading database to version 0...");
                    Upgrades.UpgradeTo0();
                }

                // 0 to 3
                if (__DBInfo.Version < 3) {
                    Console.WriteLine("Upgrading database to version 3...");
                    Upgrades.UpgradeTo3();
                }

                // 3 to 4
                if (__DBInfo.Version < 4) {
                    Console.WriteLine("Upgrading database to version 4...");
                    Upgrades.UpgradeTo4();
                }

                // 3 to 4
                if (__DBInfo.Version < 5) {
                    Console.WriteLine("Upgrading database to version 5...");
                    Upgrades.UpgradeTo5();
                }
                
            } else {
                
                // Create database
                CreateDatabase();
            }


        }
        
        /* Constructor */
        static MovieDB() 
        {
            InitFields();
            MandatoryColumns += MovieField.ID;
            MandatoryColumns += MovieField.BORROWER;
            MandatoryColumns += MovieField.BORROWEDDATE;
            MandatoryColumns += MovieField.ADDEDDATE;
        }
        
        
        
        /* Destructor */
        ~MovieDB() 
        {
            DBConnection.Close();
            DBConnection = null;		
        }
            
        
        /* Execute a Query without any return results */
        public static int ExecuteQuery(string _Query) 
        {
            IDbCommand Cmd =  DBConnection.CreateCommand();
            Cmd.CommandText = _Query;
            Cmd.ExecuteNonQuery();
            return ((SqliteCommand)Cmd).LastInsertRowID();
        }
        
        private static string GetSelectSQLQuery() 
        {
            ColumnsRequest __CR = new ColumnsRequest();
            __CR.AllColumns();
            return GetSelectSQLQuery(__CR);
            
        }

        private static string GetSelectSQLQuery(ColumnsRequest _CR) 
        {
            string __query = "SELECT ";
            
            int __i = 0;
            foreach (bool __clmn in _CR.Columns) {
                if (__clmn == true) __query += ((DataField)Fields[__i]).SqlName + ",";
                __i++;
            }
                
            __query = __query.TrimEnd(',') ;
            __query += " FROM Movies";
            
            return __query;

        }

        public static void UpdateMovie(Movie _M, ColumnsRequest _CR) 
        {
            _M.DBLize();
            
            string __query = "UPDATE Movies SET ";
            
            int __i = 0;
            foreach (bool __clmn in _CR.Columns) {
                if (__clmn == true) {
                    __query += ((DataField)Fields[__i]).SqlName + "='";
                    __query += _M.Data[__i];
                    __query += "',";
                }
                __i++;
            }
            
            __query = __query.TrimEnd(',') ;
            __query += " WHERE ID=" + _M.Data[(int)MovieField.ID];
            //Console.WriteLine(__query);
            ExecuteQuery(__query);

        }

        public static void RemoveMovie(Movie _M)
        {
            if (_M.Data[(int)MovieField.ID] == null) throw new Exception("Trying to remove a movie without ID!");
            
            string __query = "DELETE FROM Movies WHERE ID=" + _M.Data[(int)MovieField.ID]; // LIMIT 1 (FIXME: Why not works?)
            ExecuteQuery(__query);
                                                                     
                                                                      
        }

        public static Movie AddMovie(Movie _M)
        {
            Movie __InsertedMovie = _M.Clone();
            _M.DBLize();
            string __query = "INSERT INTO Movies(";
            // Sql names
            
            foreach (DataField __DF in Fields) {
                if (__DF.SqlName != "ID") __query += __DF.SqlName + ",";
                }
            
            __query = __query.TrimEnd(',');
            __query += ") VALUES (";
            
            int __l = 0;
            foreach (DataField __DF in Fields) {
                if (__DF.SqlName != "ID") __query += "'" + _M.Data[__l] + "'," ;
                __l++;
                }
            
            __query = __query.TrimEnd(',');
            __query += ")";
            
            Debug.Message("MovieDatabase.dll", "MovieDB", "Adding Movie. Query=" + __query);
            int __ID = ExecuteQuery(__query);
            __InsertedMovie.Data[(int)MovieField.ID] = Convert.ToString(__ID);
            return __InsertedMovie;
            
        }

        public static ArrayList GetCompleteColumn(string _Table, string _Element)
        {
            ArrayList _Results = new ArrayList();
 
            string __query = "SELECT " + _Element + " FROM " + _Table;
           
            IDbCommand Cmd = DBConnection.CreateCommand();
            Cmd.CommandText = __query;
            IDataReader Reader = Cmd.ExecuteReader();
            
            // Foreach row of the result
            while(Reader.Read()) 
                _Results.Add(Reader.GetString(0));

            // clean up
            Reader.Close();
            Reader = null;
            Cmd.Dispose();
            Cmd = null;

            return _Results;
        }

        public static int CheckMaxNo()
        {
            string __query = "SELECT MAX(No) FROM Movies";

            IDbCommand Cmd = DBConnection.CreateCommand();
            Cmd.CommandText = __query;
            IDataReader Reader = Cmd.ExecuteReader();
            Reader.Read();

            int __max = Convert.ToInt32(Reader.GetString(0));

            // clean up
            Reader.Close();
            Reader = null;
            Cmd.Dispose();
            Cmd = null;
                        
            //Console.WriteLine("MAX = " + __max);
            return __max;
        }
      
        
        public static Movie GetSingleRecord(string _ID)
        {
            string __query = GetSelectSQLQuery();
            __query += " WHERE ID=" + _ID;

            //Console.WriteLine(__query);
            
            // Create&Execute the command
            IDbCommand Cmd = DBConnection.CreateCommand();
            Cmd.CommandText = __query;
            IDataReader Reader = Cmd.ExecuteReader();
            
            // Read
            Reader.Read();
            
            if (Reader.FieldCount == 0) throw new Exception("Record not found! Wrong ID?");
            
            // Movie
            Movie __M = new Movie();
            for(int __i = 0; __i < Reader.FieldCount; __i++) 
                __M.Data[__i] = Reader.GetString(__i);
            
            // clean up
            Reader.Close();
            Reader = null;
            Cmd.Dispose();
            Cmd = null;
            
            return __M;
            
        }
        
       
        public delegate object ListDatabase_ProcessRecord (Movie _M, object _LastObject);
      
        /* Default wrapper */
        public static int ListDatabase(ColumnsRequest _CR, SearchRequest _SR, ListDatabase_ProcessRecord ProcFunc) 
        {
            return ListDatabase(_CR, _SR, null, ProcFunc);
        }


        /* */
        public static int ListDatabase(ColumnsRequest _CR, SearchRequest _SR, DataField _OrderBy, ListDatabase_ProcessRecord ProcFunc) 
        {
            return ListDatabase(_CR, _SR, _OrderBy, null, ProcFunc);
        }

        
        /* */
        public static int ListDatabase(ColumnsRequest _CR, SearchRequest _SR, DataField _OrderBy1, DataField _OrderBy2, ListDatabase_ProcessRecord ProcFunc) 
        {
            // First prepare the query string!
            _CR += MandatoryColumns;
            string __query = GetSelectSQLQuery(_CR);
           
            // Add Search/WHERE stuff
            if (_SR != null) __query += " " + _SR.MakeWhereQuery(Fields);

            // Order
            if (_OrderBy1 != null) {
                __query += " ORDER BY " + _OrderBy1.SqlName;
                if (_OrderBy2 != null) __query += "," + _OrderBy2.SqlName;
                __query += " ASC";
            }
                    
            // Create&Execute the command
            IDbCommand Cmd = DBConnection.CreateCommand();
            Cmd.CommandText = __query;
            IDataReader Reader = Cmd.ExecuteReader();

            object __LastObject = null;
            int __rows = 0;

            // Foreach row of the result
            while(Reader.Read()) 
                {
                    Movie __M = new Movie();
                    int __k = 0;
                    for(int __i = 0; __i < __M.Data.Length; __i++) 
                        if (_CR.Columns[__i] == true) { 
                            __M.Data[__i] = Reader.GetString(__k);
                            __k++;
                        }
                    
                    // Pass it to the list-function
                    __LastObject = ProcFunc(__M, __LastObject);
                    __M = null;                    
                    __rows++;
                }
            
            // clean up
            Reader.Close();
            Reader = null;
            Cmd.Dispose();
            Cmd = null;

            return __rows;
        }
        
        /* Completely clears the movie database */
        public static void ClearDatabase()
        {
            string __query = "DELETE FROM Movies";
            ExecuteQuery(__query);
                        
        }

    }
    
}
