/*
 * Copyright (C) 2004, 2005 Michael Dominic K. (http://www.mdk.org.pl)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using System.Collections;
using Util;

namespace MovieDatabase 
{
        
    public enum SearchComparisonType { LIKE, EQUAL, NOTEQUAL }
    public enum SearchOperand { OR, AND }
    

    public class SearchElement
    {
        public int Column;
        public SearchComparisonType Type; 
        public string Value;
        
        /* Constructor */
        public SearchElement(int _Column, SearchComparisonType _Type, string _Value)
        {
            Column = _Column;
            Type = _Type;
            Value = _Value;
        }
        
    }


    public class SearchCriterium
    {
        public ArrayList Elements = new ArrayList();
        public SearchOperand Operand;

        
        /* Constructor */
        public SearchCriterium(SearchElement _Element)
        {
            Elements.Add(_Element);
            Operand = SearchOperand.AND;
        }
        
        
        /* Constructor */
        public SearchCriterium(string _Keyword)
        {
            if ( _Keyword == String.Empty) {
                int __i = 0;
                foreach (DataField Field in MovieDB.Fields) {
                    if (Field.Searchable == true) Elements.Add(new SearchElement(__i, SearchComparisonType.EQUAL, String.Empty));
                    __i++;
                }
            }
            else {
                int __i = 0;
                foreach (DataField Field in MovieDB.Fields) {
                    if (Field.Searchable == true) Elements.Add(new SearchElement(__i, SearchComparisonType.LIKE, _Keyword));
                    __i++;
                }
            }

        }
        
    }

    
    public class SearchRequest 
    {
        ArrayList Criteriums = new ArrayList();
        SearchOperand Operand;

        /* Constructor */
        public SearchRequest( SearchElement _Element)
        {
           Criteriums.Add(new SearchCriterium(_Element));
           Operand = SearchOperand.AND;
        }
       
        
        /* Constructor */
        public void Add(SearchElement _Element)
        {
            Criteriums.Add(new SearchCriterium(_Element));
        }


        /* Empty constructor */
        public SearchRequest(SearchOperand _Operand)
        {
            Operand = _Operand;
            
        }
        
       
        /* Constructor */
        public SearchRequest(string _Keywords) 
        {
            _Keywords = Strings.ClearString(_Keywords);

            foreach (string __Key in _Keywords.Split(' ')) {
                Criteriums.Add(new SearchCriterium(__Key));
            }
            
            Operand = SearchOperand.AND;
                                 
        }
        
        
        public string MakeWhereQuery(ArrayList _Fields)
        {
            string __p = "WHERE ";
            
            foreach (SearchCriterium __SC in Criteriums) {
                __p += "(";
                foreach (SearchElement __SE in __SC.Elements) {
                
                    __p += ((DataField)_Fields[__SE.Column]).SqlName;
                    if (__SE.Type == SearchComparisonType.EQUAL) __p += "='" + __SE.Value + "'";
                    else if (__SE.Type == SearchComparisonType.LIKE) __p += " LIKE '%" + __SE.Value + "%'";
                    else if (__SE.Type == SearchComparisonType.NOTEQUAL) __p += "!='" + __SE.Value + "'";
        
                    if (__SC.Operand == SearchOperand.OR) __p += " OR ";
                    if (__SC.Operand == SearchOperand.AND) __p += " AND ";
                }
                
                if (__p.EndsWith(" OR ")) __p = __p.Remove(__p.LastIndexOf(" OR "), 4);
                if (__p.EndsWith(" AND ")) __p = __p.Remove(__p.LastIndexOf(" AND "), 5);
                __p += ")";

                if (Operand == SearchOperand.OR) __p += " OR ";
                if (Operand == SearchOperand.AND) __p += " AND ";
            }
            
            // Remove last
            if (Operand == SearchOperand.OR) __p = __p.Remove(__p.LastIndexOf(" OR "), 4);
            if (Operand == SearchOperand.AND) __p = __p.Remove(__p.LastIndexOf(" AND "), 5);
            
            Debug.Message("MovieDatabase.dll", "SearchRequest", String.Format("WHERE query is: {0}", __p));
            return __p;
        }
        
        
        
    }
    
    
}
