/*
 * Copyright (C) 2004, 2005 Michael Dominic K. (http://www.mdk.org.pl)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using System.Collections;
using System.Data;
using MovieDatabase;
using Mono.Data.SqliteClient;
using Util;

public class Upgrades {

    public static readonly string Msg_UpgradedTo4 = "Your movie database has been automatically upgraded to version 4 of Monotheka. This version introduces some changes: movies have now DATE ADDED and DATE BORROWED parameters.\n\nAll your movies were marked as being added 15 days ago. All your borrowed movies were marked as being borrowed yesterday.\n\nMonotheka will now highlight your movies as NEW and OVERBORROWED according to settings you specify. Also - you can now set DVD REGION for DVD movies.";
    
    public static bool UpgradedTo4 = false;

    //////////////////////////////////////////////////
    
    public static void UpgradeTo0()
    {
        // Create DBINfo query
        string __DBInfoQuery = "CREATE TABLE DBInfo (Name TEXT, Owner TEXT, Contact TEXT, Comment TEXT, Version TEXT)";
        string __DBInfoInsert = "INSERT INTO DBInfo (Name, Owner, Contact, Comment, Version) VALUES ('your database name', 'database owner' , 'e-mail or phone', 'additional notes' , '3')";
        
        try {
            MovieDB.ExecuteQuery(__DBInfoQuery);
            MovieDB.ExecuteQuery(__DBInfoInsert);
        } catch (Exception _Excp) {
            throw new Exception("Error during to 0 upgrading!", _Excp);
        }
        
    }
 
    /* Upgrade 2 to 3 */
    public static void UpgradeTo3()
    {
        // Create DBINfo query
        string __DBInfoUpdate = "UPDATE DBInfo SET Version = '3'";
        string __FlagsQuery = "SELECT ID, Flags FROM Movies";

        // We need to upgrade all the flags
        // Create&Execute the command
        try {
            IDbCommand Cmd = MovieDB.DBConnection.CreateCommand();
            Cmd.CommandText = __FlagsQuery;
            IDataReader Reader = Cmd.ExecuteReader();
            
            // Foreach row of the result
            while(Reader.Read()) 
                {
                    string __FlagsString = Reader.GetString(1);
                    if (__FlagsString != String.Empty) {
                        string[] __Flags = __FlagsString.Split(',');
                        string __NewFlagsString = String.Empty;
                        
                        foreach (string __Flag in __Flags) {
                            if ( __Flag != "32" ) __NewFlagsString += __Flag.PadLeft(3, '0') + ",";
                        }
                        
                        __NewFlagsString = __NewFlagsString.TrimEnd(',');
                        
                        string __AlterQuery = "UPDATE Movies SET Flags = '" + __NewFlagsString + "' WHERE ID='" + Reader.GetString(0) + "'";
                        MovieDB.ExecuteQuery(__AlterQuery);
                    }
                }
            
            // clean up
            Reader.Close();
            Reader = null;
            Cmd.Dispose();
            Cmd = null;
            
            // Now let's upgrade the info
            MovieDB.ExecuteQuery(__DBInfoUpdate);
        } catch (Exception _Excp) {
            throw new Exception("Error during to 3 upgrading!", _Excp);
        }
        
    }
    
    
    /* Upgrade 3 to 4 */
    public static void UpgradeTo4()
    {
        Debug.Message("MovieDatabase.dll", "Upgrades.cs", "Starting upgrade to version 4");

        // Create DBINfo query
        string __DBInfoUpdate = "UPDATE DBInfo SET Version = '4'";
        string __SelectMovies = "SELECT * FROM Movies";
        string __DropTable = "DROP TABLE Movies";
        string __CreateTable = "CREATE TABLE Movies (ID INTEGER PRIMARY KEY,Flags TEXT,No INT,Title TEXT,TranslatedTitle TEXT,Director TEXT,Time INT,Year INT,Country TEXT,Genre TEXT,Discs INT,VideoFormat TEXT,AudioFormat TEXT,Quality TEXT,Subtitles TEXT,Release TEXT,Borrower TEXT,Description TEXT,AddedDate TEXT,BorrowedDate TEXT,DVDRegion TEXT)"; // This is static to prevent new fields from being injected in future
                
        try {
            // Prepare query
            IDbCommand Cmd = MovieDB.DBConnection.CreateCommand();
            Cmd.CommandText = __SelectMovies;
            Debug.Message("MovieDatabase.dll", "Upgrades.cs", "Executing SELECT query to list all the movies");
            IDataReader Reader = Cmd.ExecuteReader();
            
            Debug.Message("MovieDatabase.dll", "Upgrades.cs", "Reading all the movies into memory");

            // Movies collection
            ArrayList __OldMovies = new ArrayList();
            
            // Read all the data
            while(Reader.Read()) {
                string[] __Movie = new string[18];
                
                for (int __i = 0; __i < 18; __i++)
                    __Movie[__i] = Reader.GetString(__i);
                
                // Add to collection
                __OldMovies.Add(__Movie);
            }

            // Finish the operation
            Reader.Close();
            Reader = null;
            Cmd.Dispose();
            Cmd = null;

            Debug.Message("MovieDatabase.dll", "Upgrades.cs", "Dropping old Movies Table");
            MovieDB.ExecuteQuery(__DropTable);
            
            Debug.Message("MovieDatabase.dll", "Upgrades.cs", "Creating new Movies Table");
            MovieDB.ExecuteQuery(__CreateTable);
            
            Debug.Message("MovieDatabase.dll", "Upgrades.cs", "Flushing the movies from memory to a new table");

            foreach (string[] __OldMovie in __OldMovies) {
                Movie __M = new Movie();

                // Copy all
                for (int __i = 0; __i < 18; __i++)
                    __M.Data[__i] = __OldMovie[__i];
                
                __M.Data[(int)MovieField.ADDEDDATE] = DateTime.Now.ToLocalTime().AddDays(-15).ToFileTime().ToString();
                
                // Borrower
                if (__M.Data[(int)MovieField.BORROWER] != String.Empty) __M.Data[(int)MovieField.BORROWEDDATE] = DateTime.Now.ToLocalTime().AddDays(-1).ToFileTime().ToString(); 
                else __M.Data[(int)MovieField.BORROWER] = String.Empty;
                
                __M.Data[(int)MovieField.DVDREGION] = String.Empty;

                MovieDB.AddMovie(__M);

            }
               
            Debug.Message("MovieDatabase.dll", "Upgrades.cs", "Changing DBInfo to version 4");
            MovieDB.ExecuteQuery(__DBInfoUpdate);
            UpgradedTo4 = true;
        } catch (Exception __Excp) {
            throw new Exception("Error during to 4 upgrading!", __Excp);
        }
    }


    public static void UpgradeTo5()
    {
        Debug.Message("MovieDatabase.dll", "Upgrades.cs", "Starting upgrade to version 5");
        
        // We just need to change the DB version
        string __DBInfoUpdate = "UPDATE DBInfo SET Version = '5'";

        // Execute
        Debug.Message("MovieDatabase.dll", "Upgrades.cs", "Changing DBInfo to version 5");
        MovieDB.ExecuteQuery(__DBInfoUpdate);
        
    }
                     
    


}
