/*
 * Copyright (C) 2004, 2005 Michael Dominic K. (http://www.mdk.org.pl)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* This is built on the top of GetFrom plugin, to provide a common 
 * architecture for most GetFrom plugins based on web pages (like - IMDB) 
 */

using System;
using Util;
using Util.Windows;
using Gtk;
using MovieDatabase;
using PluginLib;
using System.Collections;

namespace PluginLib {
    
    public abstract class FilmPortalPlugin : GetFromPlugin {
        
        protected delegate SearchResult AnalyzeSearchResults_Delegate(string _WWWData);
        protected delegate Movie AnalyzeMoviePage_Delegate(string _WWWData);
        
        //////////////////////////////////////////////////
        
        /* Class to hold a single result (movie) of web search */
        private class SearchResultMovie {
            public string Title = null;
            public string Link = null;
            
            
            public SearchResultMovie(string _Title, string _Link)
            {
                Title = _Title;
                Link = _Link;
            }
        }

        /* This is a window that fetches search results */
        private class FetchSearch_Window : PleaseWaitWindow {

            private string SearchedTitle;
            private string SearchAddress;
            private string Charset;
            private AnalyzeSearchResults_Delegate AnalyzeSearchResults;
            private AnalyzeMoviePage_Delegate AnalyzeMoviePage;
            
            public SearchResult Result = null;
            public Movie M = null;
           
            //////////////////////////////////////////////////
            
            /* Constructor */
            public FetchSearch_Window(string _Title, string _ServiceName, string _SearchAddress, string _Charset, AnalyzeSearchResults_Delegate _AnalyzeSearchResults, AnalyzeMoviePage_Delegate _AnalyzeMoviePage) : base(String.Format(CommonErrors.Plugins.Downloading, _ServiceName), true)
            {
                SearchAddress = _SearchAddress;
                SearchedTitle = _Title;
                Charset = _Charset;

                // Delegates

                AnalyzeSearchResults = _AnalyzeSearchResults;
                AnalyzeMoviePage = _AnalyzeMoviePage;
                
                Run();
            }
            
            
            /* Thread starter */
            public override void Start()
            {
                // Get the webpage
                string __Response = Net.GetWebpageAsString(String.Format(SearchAddress, SearchedTitle), Charset);
                SearchResult __SResults = AnalyzeSearchResults(__Response);
                
                // Moviepage
                if (__SResults.Status == SearchResultStatus.MOVIEPAGE) {
                    Debug.Message("PluginLib.dll", "FetchSearch_Window", "Movie page returned as result. Processing.");
                    M = AnalyzeMoviePage(__SResults.HTML);
                }
                
                Result = __SResults;
                
            }
            
        }
        
        /* This is a window that shows search results */
        private class SearchResults_Window : ListWindow {
            
            private const int ___Height = 300;
                        
            public SearchResultMovie SelectedMovie = null;
            
            //////////////////////////////////////////////////
            
            /* Constructor */
            public SearchResults_Window(SearchResult _Result, Gdk.Pixbuf _Logo) : base (CommonErrors.Plugins.AmbigiousTitle, CommonErrors.Plugins.SearchResultsWindowTitle, false, ___Height, _Logo)
            {

                // Fill the TreeView with results
                foreach (SearchResultMovie __SRM in _Result.Results) {
                    AddItem(Strings.ShortenString(__SRM.Title, 70), __SRM.Link);
                }
                
                ShowAll();
                Application.Run();
                   
            }
            
            
            /* Overriden cause we need Application.Quit */
            protected override void DestroyWindow()
            {
                Destroy();
                Application.Quit();
            }
            
      
            /* Process selection */
            protected override void ItemTriggered(string _1st, string _2nd)
            {
                SelectedMovie = new SearchResultMovie(_1st, _2nd);
                DestroyWindow();
            }
                        
        }

        /* This is a window that fetches movie */
        protected class FetchMovie_Window : PleaseWaitWindow {
            
            public Movie M = null;
            
            private AnalyzeMoviePage_Delegate AnalyzeMoviePage;
            private string MovieLink;
            private string Charset;
            
            //////////////////////////////////////////////////

            public FetchMovie_Window(string _Link, AnalyzeMoviePage_Delegate _AnalyzeMoviePage, string _ServiceName, string _Charset) : base(String.Format(CommonErrors.Plugins.Downloading, _ServiceName), true)
            {
                Debug.Message("PluginLib.dll", "FetchMovie_Window", "Constructing...");
                MovieLink = _Link;
                Charset = _Charset;
                AnalyzeMoviePage = _AnalyzeMoviePage;
                Run();
            }
            
            public override void Start()
            {
                Debug.Message("PluginLib.dll", "FetchMovie_Window", "Downloading data from link " + MovieLink);
                string __WWWData = Net.GetWebpageAsString(MovieLink, Charset);
                
                Debug.Message("PluginLib.dll", "FetchMovie_Window", "Starting AnalyzeMoviePage...");
                M = AnalyzeMoviePage(__WWWData);
            }
    
            
        }
    
        
        /* Enum with possible result of Search */
        public enum SearchResultStatus { RESULTS , PARTIALRESULTS, MOVIEPAGE, NOTFOUND, ERROR };
        
        
        /* This holds the results of a search */
        public class SearchResult {
            public string HTML = null;
            public ArrayList Results;
            public SearchResultStatus Status;
            
            /* Helpfull constructor */
            public SearchResult(ArrayList _Titles, ArrayList _Links)
            {
                Results = new ArrayList();
                for (int __i = 0; __i < _Titles.Count; __i++) {
                    SearchResultMovie __SRM = new SearchResultMovie((string)_Titles[__i], (string)_Links[__i]);
                    Results.Add(__SRM);
                }
                Status = SearchResultStatus.RESULTS;
            }
            
            
            /* Another one */
            public SearchResult()
            {
                Status = SearchResultStatus.NOTFOUND;
            }

            
            /* Display to console */
            public void Debug()
            {
                foreach (SearchResultMovie __SRM in Results)
                    Console.WriteLine(String.Format("TITLE: {0} LINK: {1}", __SRM.Title, __SRM.Link));
            }
            
        }
        
        //////////////////////////////////////////////////
        
        protected string BaseAddress = String.Empty;
        protected string SearchAddress = String.Empty;
        protected string Charset = String.Empty;
        protected string ServiceName = String.Empty;
        protected Gdk.Pixbuf BrandLogo = null;

        //////////////////////////////////////////////////
       
        /* Main plugin method */
        public override Movie Run(Movie _M) 
        {
            // Check if enough information
            if (_M.Data[(int)MovieField.TITLE] == null || _M.Data[(int)MovieField.TITLE] == "") {
                new Util.Windows.OKDialogWindow(CommonErrors.Plugins.NoTitle, null, MessageType.Warning);
                return null;
            }
            
            try {
                // Check if title can be properly urlized
                string __UrlizedTitle = Strings.ConvertStringFromUTF8ToURL(_M.Data[(int)MovieField.TITLE], Charset);
                if (__UrlizedTitle == null) {
                    new Util.Windows.OKDialogWindow(String.Format(CommonErrors.Plugins.BadCharset, ServiceName, Charset), null, MessageType.Warning);
                    return null;
                }
                // Start fetching
                FetchSearch_Window __FS = new FetchSearch_Window(__UrlizedTitle, ServiceName, SearchAddress, Charset, new AnalyzeSearchResults_Delegate(AnalyzeSearchResults), new AnalyzeMoviePage_Delegate(AnalyzeMoviePage));
                
                if (__FS.Result.Status == SearchResultStatus.NOTFOUND) {
                    // Movie not found
                    new Util.Windows.OKDialogWindow(String.Format(CommonErrors.Plugins.MovieNotFound, ServiceName), null, MessageType.Warning);
                    return null;
                }
                else if (__FS.M != null) return __FS.M;
                else if (__FS.Result.Status == SearchResultStatus.RESULTS || __FS.Result.Status == SearchResultStatus.PARTIALRESULTS) {
                    // Can we proceed automatically? 
                    if (__FS.Result.Status == SearchResultStatus.RESULTS && __FS.Result.Results.Count == 1) {
                        // We can download immidiately, one movie
                        FetchMovie_Window __FM = new FetchMovie_Window(((SearchResultMovie)__FS.Result.Results[0]).Link, new AnalyzeMoviePage_Delegate(AnalyzeMoviePage), ServiceName, Charset);
                        return __FM.M;
                    }
                    
                    // Results window
                    SearchResults_Window __SR = new SearchResults_Window(__FS.Result, BrandLogo);
                    if (__SR.SelectedMovie == null) return null;
                    else {
                        // Start fetching movie
                        FetchMovie_Window __FM = new FetchMovie_Window(__SR.SelectedMovie.Link, new AnalyzeMoviePage_Delegate(AnalyzeMoviePage), ServiceName, Charset);
                        return __FM.M;
                    }
                }
                
                
            } catch (ApplicationException _Excp) {
                new OKDialogWindow(_Excp.Message, null, MessageType.Error);
                return null;
          
            } catch (Exception _Excp) {
                Debug.Exception("PluginLib.dll", "FilmportalPlugin", _Excp);
                new OKDialogWindow(String.Format(CommonErrors.Plugins.Exception, _Excp.Message, _Excp.StackTrace), null, MessageType.Error);
                return null;
            }
                        
            return null; 
            
        }

        /* Delegate function */
        protected abstract SearchResult AnalyzeSearchResults(string _WWWData);
        
        /* Delefate function */
        protected abstract Movie AnalyzeMoviePage(string _WWWData);
        
        
    }


}
