/*
 * Copyright (C) 2004, 2005 Michael Dominic K. (http://www.mdk.org.pl)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using MovieDatabase;

namespace PluginLib 
{
    public enum PluginType { IMPORT, EXPORT, GETFROM };
    
    public abstract class Plugin 
    {
        public PluginType Type;

        public abstract int Initialize();
        public abstract string GetName();
        public abstract string GetAuthor();
        public abstract string GetDescription();
        
    }

    
    public abstract class ImportPlugin : Plugin
    {
        public ImportPlugin()
        {
            Type = PluginType.IMPORT;
        }
    
        public abstract bool Run();
    }

    public abstract class GetFromPlugin : Plugin
    {
        Movie M = null;
        public GetFromPlugin()
        {
            Type = PluginType.GETFROM;
        }
        
        public abstract Movie Run(Movie _M);
    }

    public abstract class ExportPlugin : Plugin
    {
        public ExportPlugin()
        {
            Type = PluginType.EXPORT;
        }

        /* Prototype. Plugin functions should be build upon that */
        public delegate void ProcessMovie(Movie _M);
        ProcessMovie MovieFuncHolder;

                
        /* Wrapper */
        public object WrapperFunc (Movie _M, object _LastObject)
        {
            MovieFuncHolder(_M);
            return null;
        }

        
        /* List Database wrapper */
        public int ListMovies(DataField _OrderBy, ProcessMovie __MovieFunc) {
            ColumnsRequest _CR = new ColumnsRequest();
            _CR.AllColumns();
            MovieFuncHolder = __MovieFunc;

            return MovieDB.ListDatabase(_CR, null, _OrderBy, new MovieDB.ListDatabase_ProcessRecord(WrapperFunc));
        }
        
        public abstract void Run();
    }
    
}
