/*
 * Copyright (C) 2004, 2005 Michael Dominic K. (http://www.mdk.org.pl)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* REV: MDK
 * DESCRIPTION: A plugin to fetch data from www.filmweb.pl - a good
 * resource of polish movies. BAsed on FilmPortal architecture.
 */

using System;
using PluginLib;
using MovieDatabase;
using Util;

public class FilmwebGet : FilmPortalPlugin {
    
    private static readonly string ___Description = "Automatically fetches some movie information from Filmweb (www.filmweb.pl). Good for polish movies. Internet connection required.";

    //////////////////////////////////////////////////

    /* Constructor */
    public FilmwebGet()
    {
        ServiceName = "Filmweb";
        Charset = "ISO-8859-2";
        SearchAddress = "http://www.filmweb.pl/Find?query={0}";
        BrandLogo = new Gdk.Pixbuf(null, "filmweb_logo.png");
    }

    /* Initialize */
    public override int Initialize()
    {
        return 0;
    }
    
    
    /* Name */
    public override string GetName() {
        return "Get from Filmweb";
    }

    
    /* Author */
    public override string GetAuthor() {
        return "Michael Dominic K. <mdk@mdk.org.pl>";
    }

        
    /* Description */
    public override string GetDescription() {
        return ___Description;
    }
    
    
    /* Analyze the WWW data containing Search Results */
    protected override SearchResult AnalyzeSearchResults(string _WWWData)
    {
        Debug.Message("FilmwebGet.dll", "FilmwebGet", "Got WWW data. Parsing search results.");
        if (_WWWData.IndexOf("szukana fraza") != -1) {
            // Search results

            Debug.Message("FilmwebGet.dll", "FilmwebGet", "This is a search results page.");
            SearchResultsPage __SRP = new SearchResultsPage(_WWWData);
            return __SRP.GetSearchResult();

        } else if (_WWWData.IndexOf("obsada") != -1) {
            // Movie Page

            Debug.Message("FilmwebGet.dll", "FilmwebGet", "This is a movie page.");
            SearchResult __SR = new SearchResult();
            __SR.Status = SearchResultStatus.MOVIEPAGE;
            __SR.HTML = _WWWData;
            return __SR;

        } else {
            Debug.Message("FilmwebGet.dll", "FilmwebGet", "This is an UNKNOWN page!");
            return null;
        }
    }

    
    /* Analyze the movie page */
    protected override Movie AnalyzeMoviePage(string _WWWData)
    {
        Debug.Message("FilmwebGet.dll", "FilmwebGet", "Got WWW data. Parsing Movie.");
        MoviePage __MP = new MoviePage(_WWWData);
        return __MP.GetMovie();
    }
    
    
}
