/*
 * Copyright (C) 2004, 2005 Michael Dominic K. (http://www.mdk.org.pl)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* REV: MDK
 * DESCRIPTION: Represents a html page containing a single movie
 * information from IMDB
 */

using System;
using MovieDatabase;
using Util;

public class MoviePage {

    string HtmlData = null;
    string HtmlDescData = null;
    
    //////////////////////////////////////////////////
    
    /* Constructor */
    public MoviePage(string _Html)
    {
        Debug.Message("IMDBGet.dll", "MoviePage", "Constructing...");
        HtmlData = _Html;

        string __PlotLink = null;
        string __Plot_Temp;

        try {
            __Plot_Temp = Tags.AfterQueryInTag("Plot Summary:", "<a href=\"", "\">", _Html);
            if (__Plot_Temp != null && __Plot_Temp != String.Empty) __PlotLink = __Plot_Temp; else 
            {
               __Plot_Temp = Strings.StartFromTill("<b class=\"ch\">Plot Outline:</b>", "<br>", _Html);
               __Plot_Temp = Strings.StartFromTill("<a href=\"", "\">", __Plot_Temp);
               __PlotLink = __Plot_Temp;
            } 
            
        } catch {
            __PlotLink = null;
        }
               
        if (__PlotLink != null && __PlotLink != String.Empty) {
            HtmlDescData = Net.GetWebpageAsString("http://www.imdb.com" + __PlotLink, "ISO-8859-1");
        } 

    }

    
    /* Get Movie information */
    public Movie GetMovie()
    {
        Debug.Message("IMDBGet.dll", "MoviePage", "Parsing HTML string data");
        
        // Title
        string __Title; 
        try {
            string __Title_Temp = Strings.StartFromTill("<h1><strong class=\"title\">", "<small>", HtmlData);
            __Title = Strings.ClearString(__Title_Temp);
        } catch {
            __Title = null;
        }

        // Director
        string __Director; 
        try {
            string __Director_Temp = Strings.StartFromTill("Directed by", "<b class=\"blackcatheader\">" , HtmlData);
            string[] __Directors = Tags.SplitTags("<a href=\"", "</a>", __Director_Temp);
            __Director_Temp = String.Empty;

            foreach (string __Direct in __Directors) {
                __Director_Temp += Strings.StartFromTill(">", null, __Direct) + ", ";
            }

            __Director_Temp = __Director_Temp.TrimEnd(',' , ' ');
            __Director = Strings.ClearString(__Director_Temp);
        } catch {
            __Director = null;
        }

        // Genre
        string __Genre; 
        try {
            string __Genre_Temp = Tags.AfterQueryInTag("Genre:", "<a href=", "</a>", HtmlData);
            __Genre_Temp = Strings.StartFromTill(">", "</a>", __Genre_Temp);
            __Genre = Strings.ClearString(__Genre_Temp);
        } catch {
            __Genre = null;
        }

        // Runtime
        string __Runtime; 
        try {
            string __Runtime_Temp = Strings.StartFromTill("Runtime:</b>", "min", HtmlData);
            __Runtime_Temp = Strings.OnlyNumbers(__Runtime_Temp);
            __Runtime = Strings.ClearString(__Runtime_Temp);
        } catch {
            __Runtime = null;
        }

        // Country
        string __Country; 
        try {

            string __Country_Temp = Strings.StartFromTill("Country:", "<br>" , HtmlData);
            string[] __Countries = Tags.SplitTags("<a href=\"", "</a>", __Country_Temp);
            __Country_Temp = String.Empty;

            foreach (string __Count in __Countries) {
                __Country_Temp += Strings.StartFromTill(">", null, __Count) + ", ";
            }

            __Country_Temp = __Country_Temp.TrimEnd(',' , ' ');
            __Country = Strings.ClearString(__Country_Temp);
        } catch {
            __Country = null;
        }
        
        // Year
        string __Year; 
        try {
            string __Year_Temp = Strings.StartFromTill("<title>" , "</title>", HtmlData);
            __Year_Temp = Strings.StartFromTill("(" , ")", __Year_Temp);
            __Year_Temp = Strings.OnlyNumbers(__Year_Temp);
            __Year = Strings.ClearString(__Year_Temp);
        } catch {
            __Year = null;
        }
        
        // Description
        string __Description; 
        try {
            if (HtmlDescData != null && HtmlDescData != String.Empty) {
                string __Description_Temp = Strings.StartFromTill("<p class=\"plotpar\">" , "</p>", HtmlDescData);
                __Description = Strings.ClearString(__Description_Temp);
            } else {
                string __Description_Temp = Strings.StartFromTill("<b class=\"ch\">Plot Outline:</b>", "<br>", HtmlData);
                __Description = Strings.ClearString(__Description_Temp);
            }
        } catch {
            __Description = null;
        }

        // Fillin
        Movie __M = new Movie();
        __M.Data[(int)MovieField.TITLE] = __Title;
        __M.Data[(int)MovieField.DIRECTOR] = __Director;
        __M.Data[(int)MovieField.GENRE] = __Genre;
        __M.Data[(int)MovieField.TIME] = __Runtime;
        __M.Data[(int)MovieField.COUNTRY] = __Country;
        __M.Data[(int)MovieField.YEAR] = __Year;
        __M.Data[(int)MovieField.DESCRIPTION] = __Description;

        return __M;
    }

}
