
/*
 * Copyright (C) 2004, 2005 Michael Dominic K. (http://www.mdk.org.pl)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using Gtk;
using Glade;
using System;
using MovieDatabase;
using PluginLib;
using System.Collections;
using Util;
using System.IO;
using System.Threading;

public class MailToEvolution : ExportPlugin {
    
    /* Widgets */
    [Widget] Gtk.Window Window_MailToEvolution_GtkWindow;
    [Widget] Gtk.Table Grid_GtkTable;

    Gtk.OptionMenu FirstSortRule_GtkOptionMenu;
    Gtk.OptionMenu SecondSortRule_GtkOptionMenu;

    /* Internal */
    ArrayList SortValues;
    
    /* Strings */
    public static string Str_Description = "Allows you to quickly send your collection to someone using Novell Evolution";

    public override int Initialize()
    {
        return 0;
    }

    public override string GetName() {
        return "Mail using Evolution";
    }

    public override string GetAuthor() {
        return "Michael Dominic K. <mdk@mdk.org.pl>";
    }

    public override string GetDescription() {
        return Str_Description;
    }

    public override void Run() {
        Glade.XML gxml = new Glade.XML (null, "MailToEvolution.glade", "Window_MailToEvolution_GtkWindow", null);
        gxml.Autoconnect (this);

        FirstSortRule_GtkOptionMenu = new Gtk.OptionMenu();
        Grid_GtkTable.Attach(FirstSortRule_GtkOptionMenu , 1, 2, 0, 1);

        SecondSortRule_GtkOptionMenu = new Gtk.OptionMenu();
        Grid_GtkTable.Attach(SecondSortRule_GtkOptionMenu , 1, 2, 1, 2);

        // Bind events

        FirstSortRule_GtkOptionMenu.Changed += new EventHandler(On_SortingRuleChanged);
        SecondSortRule_GtkOptionMenu.Changed += new EventHandler(On_SortingRuleChanged);
        
        // Menus
        
        CreatePossibleSortValues();

        // Sorting menu
        Menu __FirstSortMenu = new Menu();
        Menu __SecondSortMenu = new Menu();

        // We do this two times, since we can't re-use it 
        // (gtk-problems with parents)
        
        foreach (DataField __DF in SortValues) {
            MenuItem __MI1 = new MenuItem(__DF.Name);
            MenuItem __MI2 = new MenuItem(__DF.Name);

            __FirstSortMenu.Append(__MI1);
            __SecondSortMenu.Append(__MI2);
        }
        
        FirstSortRule_GtkOptionMenu.Menu = __FirstSortMenu;
        SecondSortRule_GtkOptionMenu.Menu = __SecondSortMenu;
        
        Window_MailToEvolution_GtkWindow.ShowAll();
        Application.Run();
    }
    

    public void CreatePossibleSortValues()
    {
        SortValues = new ArrayList();
        foreach (DataField __DF in MovieDB.Fields) {
            if (__DF.Importable == true) SortValues.Add(__DF);
        }
        // FIXME: Throw exception when no values
    }
    

    /* Delete Event */
    public void On_WindowDelete (object o, DeleteEventArgs args) 
    {
        KillWindow();
    }
    

    /* Quite obvious */
    public void KillWindow()
    {
        Window_MailToEvolution_GtkWindow.Destroy();
        Application.Quit ();
    }

    
    /* Cancel button clicked */
    public void On_Cancel (object sender, EventArgs e)
    {
        KillWindow();
    }

    
    /* Execute button clicked */
    public void On_Execute (object sender, EventArgs e)
    {
        try {

            string __TempFilename = Path.GetTempPath() + DateTime.Now.ToFileTime();

            Debug.Message("TXTExport.dll", "MailToEvolution", String.Format("Writing {0} temporary file", __TempFilename));
            TextWriter __Wr = new TextWriter(__TempFilename);

            ColumnsRequest _CR = new ColumnsRequest();
            _CR.AllColumns();
            MovieDB.ListDatabase(_CR, null, (DataField)SortValues[FirstSortRule_GtkOptionMenu.History], (DataField)SortValues[SecondSortRule_GtkOptionMenu.History], new MovieDB.ListDatabase_ProcessRecord(__Wr.WriteMovie));

            __Wr.CloseFile();

            Debug.Message("TXTExport.dll", "MailToEvolution", String.Format("Closing {0}", __TempFilename));

            // Let's read the file
            // FIXME Writing file to disk is USELESS!
            string __CreatedFileContents = Util.Strings.ReadFromFile(__TempFilename);
            __CreatedFileContents = __CreatedFileContents.Replace("&", "%26");

            Debug.Message("TXTExport.dll", "MailToEvolution", "Starting Novell Evolution 2.2");
            // Launch evolution
                try {
                    System.Diagnostics.Process proc = new System.Diagnostics.Process(); 
                    proc.EnableRaisingEvents = false; 
                    proc.StartInfo.FileName = "evolution-2.2"; 
                    proc.StartInfo.Arguments = "\"mailto:?body=" + __CreatedFileContents + "\"";
                    proc.Start(); 
                    // FIXME We need to check if evolution was actually started
                }
                catch {
                    throw new Exception("Could not start evolution!");
                }

                //Delete temporary file
                Debug.Message("TXTExport.dll", "MailToEvolution", String.Format("Deleting {0}", __TempFilename));
                File.Delete(__TempFilename);
                
                KillWindow();
                        
        } catch (Exception __Excp) {
            // Fallback
            new Util.Windows.OKDialogWindow(__Excp.Message, Window_MailToEvolution_GtkWindow, MessageType.Error);
            Debug.Exception("TXTExport.dll", "MailToEvolution", __Excp);
           
            return;
        }


    }


    /* User changed one of the sorting rules. We need to make
     * sure that sorting rules are not equal */
    public void On_SortingRuleChanged(object _O, EventArgs _Args)
    {
        Gtk.OptionMenu __JustChanged = (Gtk.OptionMenu) _O;
        Gtk.OptionMenu __TheOtherOne;

        if ( __JustChanged == FirstSortRule_GtkOptionMenu) __TheOtherOne = SecondSortRule_GtkOptionMenu;
            else __TheOtherOne = FirstSortRule_GtkOptionMenu;

        // Conflicting situation
        if ( __JustChanged.History == __TheOtherOne.History) {
            uint __SubstituteWith;
            if (__JustChanged.History < SortValues.Count - 1) __SubstituteWith = (uint) __JustChanged.History + 1;
                else __SubstituteWith = 0;

            __TheOtherOne.SetHistory(__SubstituteWith);
        }

            
    }

    
}
