/*
 * Copyright (C) 2004, 2005 Michael Dominic K. (http://www.mdk.org.pl)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using Gtk;
using Glade;
using System.Threading;

namespace Util.Windows {

    public abstract class PleaseWaitWindow : Window {
        
        [Widget] protected Window Window_PleaseWait_GtkWindow;
        [Widget] protected ProgressBar Pulse_GtkProgressBar;
        [Widget] protected Label Message_GtkLabel;
        [Widget] protected Button Abort_GtkButton;
        
        protected uint TimeoutID;
        protected Thread ControlledT;
        protected bool ToBeTerminated = false;
        protected bool Abortable = false;
        

        public bool Aborted = false;
        Exception Excp = null;
        ApplicationException AExcp = null;

        /* Constructor */
        public PleaseWaitWindow(string _Msg, bool _AbortPossible) : base (IntPtr.Zero)
        {
            Glade.XML gxml = new Glade.XML (null, "PleaseWaitWindow.glade", "Window_PleaseWait_GtkWindow", null);
            gxml.Autoconnect (this);

            Raw = Window_PleaseWait_GtkWindow.Handle;
            
            // Abort button
            Message_GtkLabel.Text = _Msg;
            if (_AbortPossible) { 
                Abort_GtkButton.Visible = true;
                Abort_GtkButton.Clicked += new EventHandler(On_Abort);
                Abortable = true;
            } else {
                Abort_GtkButton.Visible = false;
                Abort_GtkButton.Hide();
                Abort_GtkButton.Destroy();
            }
            
                      
            
        }
        
        /* Main function */
        public abstract void Start();

        public void ThreadStarter()
        {
            try {
                Start();
            } catch (ApplicationException __AExcp) {
                Util.Debug.Message("Util.dll", "PleaseWaitWindow", "ApplicationException in thread! Throwing back...");
                AExcp = __AExcp;
            } catch (Exception __Excp) {
                Util.Debug.Message("Util.dll", "PleaseWaitWindow", "Exception in thread! Throwing back...");
                Excp = __Excp;
             } finally {
                ToBeTerminated = true;
            }
        }

        /* That needs to be run by the overriden constructor */
        public void Run()
        {
            // Create thread
            ControlledT = new Thread(new ThreadStart(ThreadStarter));
            ControlledT.Start();
            
            if (!ToBeTerminated) {
                // Register the timeout function
                TimeoutID = Gtk.Timeout.Add(50, new Function(Pulse));
                                
                ShowAll();
                Gtk.Application.Run();
            }

            // Exception happened in thread
            if (AExcp != null) throw AExcp; else
                if (Excp != null) throw Excp;
        }

        
        public void On_Abort (object o, EventArgs args)
        {
            Aborted = true;
            try {
                ControlledT.Abort();
            } catch {
            }
                
            DestroyWindow();
        }

        
        public virtual void DestroyWindow()
        {
            Gtk.Timeout.Remove(TimeoutID);
            Destroy();
            Gtk.Application.Quit();
        }
        

        /* Pulsate the progress bar */
        private bool Pulse()
        {
            if (ToBeTerminated) DestroyWindow();
            else Pulse_GtkProgressBar.Pulse();
            return true;
        }
        

        /* Same as pressing cancel if abortable. Otherwise does nothing */
        public void On_WindowDelete (object o, DeleteEventArgs args) 
        {
            if (Abortable) On_Abort(this, null);
            args.RetVal = true;
        }


    }


}
