/*
 * Copyright (C) 2004, 2005 Michael Dominic K. (http://www.mdk.org.pl)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* Set of functions to load & parse .rc files. The format of the files 
 * is bash-like. Ex:
 *
 * NAME=This is a test
 * DESCRIPTION=This is a description
 * # This is a comment
 * 
 */

using System;
using System.IO;
using System.Collections;

namespace Util {

    public class RCConfigFile 
    {

        public class Variable 
        {
            public string Name;
            public string Value;

            /* Constructor */
            public Variable (string _Name, string _Value)
            {
                Name = _Name;
                Value = _Value;
            }

        }

        ArrayList Variables = new ArrayList();

        //////////////////////////////////////////////////

        /* Constructor */
        public RCConfigFile(string _Filename)
        {
            StreamReader __Reader = File.OpenText(_Filename);

            string __CurrentLine = null;

            // FIXME We should move all the trim start and trim end to clear-string like
            // function, that DOESN'T delete spaces inside the string.

            while ((__CurrentLine = __Reader.ReadLine()) != null) {

                // Clear the string
                __CurrentLine = __CurrentLine.TrimStart(' ', '\t');
                __CurrentLine = __CurrentLine.TrimEnd(' ', '\t');

                // Check, if this is a comment
                if (! __CurrentLine.StartsWith("#")) {

                    // Now we can analyze that...
                    int __EqualSign = __CurrentLine.IndexOf('=');
                    if (__EqualSign == -1) throw new Exception("Broken .rc file!");

                    // Name of the variable...
                    string __Name = __CurrentLine.Substring(0, __EqualSign);
                    __Name = __Name.TrimEnd(' ', '\t');

                    // Value of the variable...
                    // FIXME Out of range exception
                    string __Value = __CurrentLine.Substring(__EqualSign+1);
                    __Value = __Value.TrimStart(' ', '\t');

                    Variable __V = new Variable (__Name, __Value);
                    Variables.Add(__V);                
                }

                // We just skip comment lines
            }

            __Reader.Close();
        }


        /* Get a variable value */
        public string GetValue(string _VariableName)
        {
            foreach (Variable _V in Variables)
                if (_V.Name == _VariableName) return _V.Value;

            // Not found
            return null;
        }


        /* Checks if a given variable is declared */
        public bool IsVariableDeclared(string _VariableName)
        {
            foreach (Variable _V in Variables)
                if (_V.Name == _VariableName) return true;

            return false;
        }


        /* Print all the variables/values to the console */
        public void Debug()
        {
            Console.WriteLine("RC FILE CONTENTS:");

            foreach (Variable __V in Variables) {
                Console.WriteLine("   {0}:{1}", __V.Name, __V.Value);
            }


        }
    }

}
