/*
 * Copyright (C) 2004, 2005 Michael Dominic K. (http://www.mdk.org.pl)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using System.Text.RegularExpressions;
using System.Text;
using System.Runtime.InteropServices;
using System.IO;

namespace Util {

    public class UnManagedCodeException : ApplicationException {
        
        public UnManagedCodeException(string _Msg) : base (_Msg)
        {
        }
        
    }


    public class Strings {

        /* Read an entire (text) file to a string */
        public static string ReadFromFile(string _Filename)
        {
            StreamReader __SR = new StreamReader(_Filename);
            string __Contents = __SR.ReadToEnd();
            __SR.Close();
            return __Contents;
        }
        

        /* Get a string UTF8 containing one character, which is _Entitya 
           in Charset */
        public static string OneByteEntityToUTF8(byte _Entity, string _Charset)
        {
            byte[] __Str = new byte[1];
            __Str[0] = _Entity;
            
            return ANSIBytesToUTF8String(__Str, _Charset);
        }
            
        
        /* Convert a set of bytes to UTF8 */
        public static string ANSIBytesToUTF8String(byte[] _Data, string _Charset)
        {
            Encoding __Enc = System.Text.Encoding.GetEncoding(_Charset);
            return __Enc.GetString(_Data);
        }


        /* Turn UTF8 string into array of bytes */
        public static byte[] DecodeUTF8String(string _Str)
	{
		UTF8Encoding UTF8 = new UTF8Encoding();
		return UTF8.GetBytes(_Str);
	}

        
        
        
        public static string EntitizeUrlStyle(byte[] _Bytes)
	{
		string __Url = String.Empty;
		foreach (byte __Byte in _Bytes) {
			if (__Byte >= 128) __Url += String.Format("%{0:x}" , __Byte);
			else __Url += (char) __Byte;
		}
		
		return __Url;
	}



        /* Convert a string from one charset to other */
        public static string ConvertStringFromUTF8ToURL(string _Str, string _ToCharset)
        {
            byte[] __BytesArray = DecodeUTF8String(_Str);
            Encoding __SrcEnc = System.Text.Encoding.GetEncoding("UTF-8");
            Encoding __DestEnc = System.Text.Encoding.GetEncoding(_ToCharset);

            byte[] __ConvertedBytes = Encoding.Convert(__SrcEnc, __DestEnc, __BytesArray);
            return EntitizeUrlStyle(__ConvertedBytes);
            
        }
            
        
        /* Converts an array of bytes to a string ASCII MODE */ 
        public static string ASCIIBytesToString(byte[] _Data)
        {
            ASCIIEncoding __Encoding = new ASCIIEncoding();
            string __S = __Encoding.GetString(_Data);
            __Encoding = null;
            return __S; 
        }
        
        
        /* Converts an array of bytes to a string UNICODE MODE */ 
        public static string UnicodeBytesToString(byte[] _Data)
        {
            UnicodeEncoding __Encoding = new UnicodeEncoding();
            string __S = __Encoding.GetString(_Data);
            __Encoding = null;
            return __S; 
        }

        
        /* Makes a string DB-compatible */
        public static string DBLize(string _S)
        {
            _S = _S.Replace("'", "''");
            return _S;

        }


        /* If the string exceeds _MaxLength, it get's trimmed (BLah Blah...) */
        public static string ShortenString(string _Text, int _MaxLength)
        {
            if (_Text.Length <= _MaxLength) return _Text;
            
            return _Text.Substring(0, _MaxLength - 3) + "...";
        }


        /* Clear string from all non-numerical data */
        public static string OnlyNumbers(string _Text)
        {
            string __NewString = String.Empty;

            foreach (char __c in _Text.ToCharArray())
                if (Char.IsDigit(__c)) __NewString += __c;

            return __NewString;
        }

        
        /* Center the text */
        public static string MakeTitle(string _Text, int _Columns, bool _Capitalize)
        {
            if ( _Text.Length >= _Columns) return _Text;

            int __LenNeeded = (_Columns - _Text.Length) / 2 + _Text.Length;
            string __Header = _Text ;

            __Header = __Header.PadLeft(__LenNeeded);
            if ( _Capitalize) __Header = __Header.ToUpper();
            return __Header;
            
        }


        /* Remove HTML <> markup */
        public static string StripMarkup(string _Text) 
        {
            string __Text = _Text.Replace("<br>", "\n");
            return Regex.Replace(__Text,@"<(.|\n)*?>", string.Empty);
        }

        
        /* Strip content inside ( ) */
        public static string StripParenthesis(string _Text)
        {
            return Regex.Replace(_Text,@"\((.|\n)*?\)", string.Empty);
        }

        
        /* Remove HTML <> markup with some optimization 
           towards the dataview                        */
        public static string StripMarkupFaster(string _Text) 
        {
            if (_Text == null) return null;
            if (_Text.StartsWith("<")) return StripMarkup(_Text);
            else return _Text;
        }
        

        
        /* Make the string HTML/Markup compatible */
        public static string MarkupizeText(string _Text)
        {
            string _Res = _Text.Replace("&", "&amp;");
            _Res = _Res.Replace("<" , "&lt;");
            _Res = _Res.Replace(">" , "&gt;");
            return _Res;
        }


        /* Make the string HTML/Markup compatible + change newlines to BR */
        public static string MarkupizeTextBR(string _Text)
        {
            string _Res = _Text.Replace("&", "&amp;");
            _Res = _Res.Replace("<" , "&lt;");
            _Res = _Res.Replace(">" , "&gt;");
            _Res = _Res.Replace("\n" , "<br>");
            return _Res;
        }


        /* Gets the string starting from Start till matching End */
        public static string StartFromTill (string _Start, string _End, string _Data)
        {
            int __StartPos;

            if ( _Start != null) {
                __StartPos = _Data.IndexOf(_Start);
                if (__StartPos == -1) return null;
                else __StartPos = __StartPos + _Start.Length;
            }
            else __StartPos = 0;

            int __EndPos;
            
            if (_End != null) __EndPos = _Data.IndexOf(_End, __StartPos);
                else __EndPos = _Data.Length;
                    
            if ( __EndPos == -1) __EndPos = _Data.Length;
            return _Data.Substring(__StartPos, __EndPos - __StartPos);
        }

        
        /* Remove unneeded spaces, etc. */
        public static string ClearString(string _S)
        {
            string __R = _S;
            __R = __R.TrimStart(' ', '\t' ,  '\n', (char) 0xa0);
            __R = __R.TrimEnd(' ', '\t' , '\n', (char) 0xa0);
            // 0xa0 is a non-breaking space (whatever)
            
            // FIXME: We should do some regexing here...

            __R = __R.Replace("  ", " "); // Some spaces replacing
            __R = __R.Replace("  ", " ");
            __R = __R.Replace("  ", " ");

            __R = __R.Replace(",  ", ", ");
            __R = __R.Replace(",  ", ", ");
            __R = __R.Replace(" , ", ", ");
            
            return __R;
        }

        
        /* Join all the lines of a string */
        public static string NoNewLines(string _S)
        {
            _S = _S.Replace('\n', ' ');
            return _S;
        }
       

        /* Return an array containing all the availible encodings */
        public static string[] GetEncodings()
        {
            string[] __Encodings = { "UTF-8", "ISO8859-2", "ISO8859-1" };
            return __Encodings;
        }
        
    }


}
