/*
 * Copyright (C) 2004, 2005 Michael Dominic K. (http://www.mdk.org.pl)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using System.Collections;
using Util;
using System.Globalization;

namespace Util {

    public class Tags {
        
        /* Returns the contents of the first given tag AFTER the query */
        public static string AfterQueryInTag(string _Query, string _StartTag, string _EndTag, string _Data)
        {
            // First let's find the query 
            int __Pos = _Data.IndexOf(_Query);
            if (__Pos == -1) return null;
            __Pos += _Query.Length;
            
            // Now, let's find the start tag position
            int __StartTagPos = _Data.IndexOf(_StartTag, __Pos);
            if (__StartTagPos == -1) return null;
            __StartTagPos +=  _StartTag.Length;
            
            // Now, let's find the end tag position
            int __EndTagPos = _Data.IndexOf(_EndTag, __StartTagPos);
            if (__EndTagPos == -1) return null;
            
            string __Ret = _Data.Substring(__StartTagPos, __EndTagPos - __StartTagPos);
            return __Ret;
        }

        
        /* Returns an array containing contents of all the indexes of 
           the specified tag. Kind of hard to explain                   */
        public static string[] SplitTags(string _TagStart, string _TagEnd, string _Data)
        {
            int __Enum = 0;
            
            int __StartPos;
            int __EndPos;
            
            ArrayList __TagList = new ArrayList();
            
            while ((__StartPos = _Data.IndexOf(_TagStart, __Enum)) != -1 ) {
                __StartPos += _TagStart.Length;
                __EndPos = _Data.IndexOf(_TagEnd, __StartPos);
                if (__EndPos == -1) break;
                
                string __Current = _Data.Substring(__StartPos, __EndPos - __StartPos);
                __TagList.Add(__Current);
                __Enum = __EndPos + _TagEnd.Length;
            }
            
            if ( __TagList.Count == 0) return null;
            
            return (string[]) __TagList.ToArray(typeof(string));
        }


        /* Parses the HTML entities in a string converting them 
           to valid unicode stuff */
        public static string RemoveHTMLEntities(string _Data, string _Charset)
        {
            
            int __StartPos;
            int __EndPos;
            
            // FIXME: We need some protection against &amp;
            while ((__StartPos = _Data.IndexOf("&#")) != -1 ) {
                __StartPos += 2;
                __EndPos = _Data.IndexOf(';', __StartPos);
                if (__EndPos == -1) break;
                
                string __Entity = _Data.Substring(__StartPos, __EndPos - __StartPos);
                
                int __Entity_Num;

                if (__Entity.StartsWith("x")) {
                    string __Entity_Hex = _Data.Substring(__StartPos+1, __EndPos - __StartPos-1);
                    __Entity_Num = Int32.Parse(__Entity_Hex, NumberStyles.HexNumber); 
                } else {
                    __Entity_Num = Convert.ToInt32(__Entity); 
                }
                
                // FIXME: What this (Entity > 255) means? 

                string __OneChar;
                if (__Entity_Num <= 255) __OneChar = Strings.OneByteEntityToUTF8((byte)__Entity_Num, _Charset);
                else __OneChar = String.Empty;

                _Data = _Data.Replace("&#" + __Entity + ";" , __OneChar);
                

            }
           
        
            return _Data;
        }

        
        
        



    }

}
